/*
 * GNOME hardware sensors panel applet
 * (C) 1999 The Free Software Foundation
 *
 * Displays the state of hardware sensors in your machine.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *                                
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *           
 * Authors: 	Brendon Humphrey (brendy@swipnet.se)
 *
 * This applet is based on the diskusage applet and the 'sensors'
 * application that is included in the lm_sensors package.
 *
 */

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>
#include <time.h>
#include <gnome.h>
#include <assert.h>
#include <gdk/gdkx.h>
#include <sensors/sensors.h>
#include <sensors/error.h>
#include <gdk_imlib.h>

#include "../config.h"
#include "gnomesensors.h"

/* cause a compile error if HAS_SENSORS is not defined by autoconf */
#ifndef HAS_SENSORS
#error Please install lm_sensors, then execute configure again.
#endif


/* ==========================================================================
 * globals
 * =========================================================================*/
GnomeSensorsApp *sensors_app;


/* ***************************************************************************************************************
 *  Start of Value Font Picker Code.
 * **************************************************************************************************************/


/* ==========================================================================
 * on_value_font_delete
 * =========================================================================*/
static gint on_value_font_delete( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->value_font_picker);
	return TRUE;
}

/* ==========================================================================
* on_value_font_ok
* =========================================================================*/
void on_value_font_ok( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	if( sensors_app->temporary_properties.value_font != NULL )
	{
		g_free(sensors_app->temporary_properties.value_font);
	}
	
	sensors_app->temporary_properties.value_font = g_strdup(gtk_font_selection_dialog_get_font_name
			(GTK_FONT_SELECTION_DIALOG (sensors_app->value_font_picker)));
	
	gtk_widget_destroy(sensors_app->value_font_picker);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_value_font_cancel
* =========================================================================*/
void on_value_font_cancel( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->value_font_picker);
}

/* ==========================================================================
 * on_pb_value_font_clicked
 * =========================================================================*/
void on_pb_value_font_clicked( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	sensors_app->value_font_picker = gtk_font_selection_dialog_new (_("Select a Units font..."));
	
	gtk_font_selection_dialog_set_font_name(
		GTK_FONT_SELECTION_DIALOG(sensors_app->value_font_picker),
		sensors_app->properties.value_font);
	
	gtk_window_set_title (GTK_WINDOW (sensors_app->value_font_picker), _("Select a Units font..."));
	gtk_window_set_policy (GTK_WINDOW (sensors_app->value_font_picker), FALSE, TRUE, TRUE);
	gtk_window_position (GTK_WINDOW (sensors_app->value_font_picker), GTK_WIN_POS_MOUSE);
	gtk_widget_set (GTK_WIDGET (sensors_app->value_font_picker), 
		"GtkWindow::auto_shrink", FALSE,
		NULL);  
	
	/* handle the wm close signal */
	gtk_signal_connect (
		GTK_OBJECT (sensors_app->value_font_picker), 
		"delete_event",
		GTK_SIGNAL_FUNC (on_value_font_delete),
		sensors_app);
	
  	/* ok and cancel buttons */
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->value_font_picker)->ok_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_value_font_ok),
		sensors_app);
	
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->value_font_picker)->cancel_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_value_font_cancel),
		sensors_app);
	
	gtk_widget_show (sensors_app->value_font_picker);
}


/* ***************************************************************************************************************
 *  Start of Units Font Picker Code.
 * **************************************************************************************************************/


/* ==========================================================================
 * on_units_font_delete
 * =========================================================================*/
static gint on_units_font_delete( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->units_font_picker);
	return TRUE;
}

/* ==========================================================================
* on_units_font_ok
* =========================================================================*/
void on_units_font_ok( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	if( sensors_app->temporary_properties.units_font != NULL )
	{
		g_free(sensors_app->temporary_properties.units_font);
	}
	
	sensors_app->temporary_properties.units_font = g_strdup(gtk_font_selection_dialog_get_font_name
			(GTK_FONT_SELECTION_DIALOG (sensors_app->units_font_picker)));
	
	gtk_widget_destroy(sensors_app->units_font_picker);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_units_font_cancel
* =========================================================================*/
void on_units_font_cancel( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->units_font_picker);
}

/* ==========================================================================
 * on_pb_units_font_clicked
 * =========================================================================*/
void on_pb_units_font_clicked( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	sensors_app->units_font_picker = gtk_font_selection_dialog_new (_("Select a Units font..."));
	
	gtk_font_selection_dialog_set_font_name(
		GTK_FONT_SELECTION_DIALOG(sensors_app->units_font_picker),
		sensors_app->properties.units_font);
	
	gtk_window_set_title (GTK_WINDOW (sensors_app->units_font_picker), _("Select a Units font..."));
	gtk_window_set_policy (GTK_WINDOW (sensors_app->units_font_picker), FALSE, TRUE, TRUE);
	gtk_window_position (GTK_WINDOW (sensors_app->units_font_picker), GTK_WIN_POS_MOUSE);
	gtk_widget_set (GTK_WIDGET (sensors_app->units_font_picker), 
		"GtkWindow::auto_shrink", FALSE,
		NULL);  
	
	/* handle the wm close signal */
	gtk_signal_connect (
		GTK_OBJECT (sensors_app->units_font_picker), 
		"delete_event",
		GTK_SIGNAL_FUNC (on_units_font_delete),
		sensors_app);
	
  	/* ok and cancel buttons */
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->units_font_picker)->ok_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_units_font_ok),
		sensors_app);
	
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->units_font_picker)->cancel_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_units_font_cancel),
		sensors_app);
	
	gtk_widget_show (sensors_app->units_font_picker);
}


/* ***************************************************************************************************************
 *  Start of Label Font Picker Code.
 * **************************************************************************************************************/


/* ==========================================================================
 * on_label_font_delete
 * =========================================================================*/
static gint on_label_font_delete( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->label_font_picker);
	return TRUE;
}

/* ==========================================================================
* on_label_font_ok
* =========================================================================*/
void on_label_font_ok( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	if( sensors_app->temporary_properties.label_font != NULL )
	{
		g_free(sensors_app->temporary_properties.label_font);
	}
	
	sensors_app->temporary_properties.label_font = g_strdup(gtk_font_selection_dialog_get_font_name
			(GTK_FONT_SELECTION_DIALOG (sensors_app->label_font_picker)));
	
	gtk_widget_destroy(sensors_app->label_font_picker);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_label_font_delete
* =========================================================================*/
void on_label_font_cancel( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	gtk_widget_destroy(sensors_app->label_font_picker);
}

/* ==========================================================================
 * on_pb_label_font_clicked
 * =========================================================================*/
void on_pb_label_font_clicked( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	sensors_app->label_font_picker = gtk_font_selection_dialog_new (_("Select a label font..."));
	
	gtk_font_selection_dialog_set_font_name(
		GTK_FONT_SELECTION_DIALOG(sensors_app->label_font_picker),
		sensors_app->properties.label_font);
	
	gtk_window_set_title (GTK_WINDOW (sensors_app->label_font_picker), _("Select a label font..."));
	gtk_window_set_policy (GTK_WINDOW (sensors_app->label_font_picker), FALSE, TRUE, TRUE);
	gtk_window_position (GTK_WINDOW (sensors_app->label_font_picker), GTK_WIN_POS_MOUSE);
	gtk_widget_set (GTK_WIDGET (sensors_app->label_font_picker), 
		"GtkWindow::auto_shrink", FALSE,
		NULL);  
	
	/* handle the wm close signal */
	gtk_signal_connect (
		GTK_OBJECT (sensors_app->label_font_picker), 
		"delete_event",
		GTK_SIGNAL_FUNC (on_label_font_delete),
		sensors_app);
	
  	/* ok and cancel buttons */
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->label_font_picker)->ok_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_label_font_ok),
		sensors_app);
	
	gtk_signal_connect (
		GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(sensors_app->label_font_picker)->cancel_button),
		"clicked", 
		GTK_SIGNAL_FUNC(on_label_font_cancel),
		sensors_app);
	
	gtk_widget_show (sensors_app->label_font_picker);
}


/* ***************************************************************************************************************
 *  Properties dialog code
 * **************************************************************************************************************/

/* ==========================================================================
* on_width_change
* =========================================================================*/
void on_width_change( GtkAdjustment *width, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.width = width->value;	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_height_change
* =========================================================================*/
void on_height_change( GtkAdjustment *height, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.height = height->value;	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_label_y_change
* =========================================================================*/
void on_label_y_change( GtkAdjustment *label_y, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.name_ypos = label_y->value;	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_value_y_change
* =========================================================================*/
void on_value_y_change( GtkAdjustment *value_y, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.value_ypos = value_y->value;	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_units_y_change
* =========================================================================*/
void on_units_y_change( GtkAdjustment *units_y, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.units_ypos = units_y->value;
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_interval_change
* =========================================================================*/
void on_interval_change( GtkAdjustment *interval, GnomeSensorsApp *sensors_app)
{
	sensors_app->temporary_properties.speed = 1000 * interval->value;
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sensors_app->property_box));
}

/* ==========================================================================
* on_pb_apply_clicked
* =========================================================================*/
void on_pb_apply_clicked( GtkWidget *widget, gint page, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	save_properties(
		APPLET_WIDGET(sensors_app->my_applet)->privcfgpath,
		&sensors_app->temporary_properties);
	
	load_properties(
		APPLET_WIDGET(sensors_app->my_applet)->privcfgpath,
		&sensors_app->properties);
	
	sensors_resize(sensors_app);
	sensors_change_fonts(sensors_app);
	start_timer(sensors_app);	
}

/* ==========================================================================
 * on_pb_cancel_clicked
 * =========================================================================*/
void on_pb_cancel_clicked( GtkWidget *widget, void *data )
{
	printf("cancel\n");
}

/* ==========================================================================
 * on_destroy_properties
 * =========================================================================*/
void on_destroy_properties( GtkWidget *widget, void *data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	sensors_app->property_box = NULL;
}

/* ==========================================================================
 * create_first_page
 * =========================================================================*/
GtkWidget *create_first_page(GnomeSensorsApp *sensors_app)
{
	sensors_properties *properties = &sensors_app->properties;
	GtkWidget *update_interval;
	GtkWidget *hbox8;
	GtkWidget *vbox4;
	GtkWidget *fonts_frame;
	GtkWidget *vbox9;
	GtkWidget *hbox24;
	GtkWidget *txt_label_font;
	GtkWidget *pb_label_font;
	GtkWidget *hbox25;
	GtkWidget *txt_value_font;
	GtkWidget *pb_value_font;
	GtkWidget *hbox26;
	GtkWidget *txt_utils_font;
	GtkWidget *pb_units_font;
	GtkWidget *dimensions_frame;
	GtkWidget *vbox5;
	GtkWidget *hbox14;
	GtkWidget *txt_width;
	GtkWidget *width;
	GtkWidget *hbox15;
	GtkWidget *txt_height;
	GtkWidget *height;
	GtkWidget *hbox16;
	GtkWidget *txt_labley;
	GtkWidget *label_y_pos;
	GtkWidget *hbox17;
	GtkWidget *txt_valuey;
	GtkWidget *value_y_pos;
	GtkWidget *hbox18;
	GtkWidget *txt_unitsy;
	GtkWidget *units_y_pos;
	GtkWidget *frame10;
	GtkWidget *hbox27;
	GtkWidget *txt_update_interval;
	GtkObject *width_adj;
	GtkObject *height_adj;
	GtkObject *label_y_adj;
	GtkObject *value_y_adj;
	GtkObject *units_y_adj;
	GtkObject *interval_adj;
	
	hbox8 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox8);
	gtk_container_border_width (GTK_CONTAINER (hbox8), 10);
	
	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox4);
	gtk_box_pack_start (GTK_BOX (hbox8), vbox4, TRUE, TRUE, 0);
	
	fonts_frame = gtk_frame_new ("Fonts");
	gtk_widget_show (fonts_frame);
	gtk_box_pack_start (GTK_BOX (vbox4), fonts_frame, TRUE, TRUE, 5);
	
	vbox9 = gtk_vbox_new (FALSE, 5);
	gtk_widget_show (vbox9);
	gtk_container_add (GTK_CONTAINER (fonts_frame), vbox9);
	gtk_container_border_width (GTK_CONTAINER (vbox9), 5);
	
	hbox24 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox24);
	gtk_box_pack_start (GTK_BOX (vbox9), hbox24, TRUE, TRUE, 0);
	
	txt_label_font = gtk_label_new ("Label Font:");
	gtk_widget_show (txt_label_font);
	gtk_box_pack_start (GTK_BOX (hbox24), txt_label_font, FALSE, TRUE, 0);
	
	pb_label_font = gtk_button_new_with_label ("Set...");
	gtk_widget_show (pb_label_font);
	gtk_box_pack_start (GTK_BOX (hbox24), pb_label_font, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (pb_label_font), 2);
	gtk_signal_connect (GTK_OBJECT (pb_label_font), "clicked",
		GTK_SIGNAL_FUNC (on_pb_label_font_clicked),
		sensors_app);
	
	hbox25 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox25);
	gtk_box_pack_start (GTK_BOX (vbox9), hbox25, TRUE, TRUE, 0);
	
	txt_value_font = gtk_label_new ("Value Font:");
	gtk_widget_show (txt_value_font);
	gtk_box_pack_start (GTK_BOX (hbox25), txt_value_font, FALSE, TRUE, 0);
	
	pb_value_font = gtk_button_new_with_label ("Set...");
	gtk_widget_show (pb_value_font);
	gtk_box_pack_start (GTK_BOX (hbox25), pb_value_font, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (pb_value_font), 2);
	gtk_signal_connect (GTK_OBJECT (pb_value_font), "clicked",
		GTK_SIGNAL_FUNC (on_pb_value_font_clicked),
		sensors_app);
	
	hbox26 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox26);
	gtk_box_pack_start (GTK_BOX (vbox9), hbox26, TRUE, TRUE, 0);
	
	txt_utils_font = gtk_label_new ("Units Font:");
	gtk_widget_show (txt_utils_font);
	gtk_box_pack_start (GTK_BOX (hbox26), txt_utils_font, FALSE, TRUE, 0);
	
	pb_units_font = gtk_button_new_with_label ("Set...");
	gtk_widget_show (pb_units_font);
	gtk_box_pack_start (GTK_BOX (hbox26), pb_units_font, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (pb_units_font), 2);
	gtk_signal_connect (GTK_OBJECT (pb_units_font), "clicked",
		GTK_SIGNAL_FUNC (on_pb_units_font_clicked),
		sensors_app);
	
	dimensions_frame = gtk_frame_new ("Dimensions");
	gtk_widget_show (dimensions_frame);
	gtk_box_pack_start (GTK_BOX (vbox4), dimensions_frame, TRUE, TRUE, 5);
	
	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);
	gtk_container_add (GTK_CONTAINER (dimensions_frame), vbox5);
	gtk_container_border_width (GTK_CONTAINER (vbox5), 5);
	
	hbox14 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox14);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox14, TRUE, TRUE, 0);
	
	txt_width = gtk_label_new ("Width:");
	gtk_widget_show (txt_width);
	gtk_box_pack_start (GTK_BOX (hbox14), txt_width, TRUE, TRUE, 0);
	
	width_adj = gtk_adjustment_new (properties->width, 30, 80, 1, 1, 1);
	width = gtk_hscale_new (GTK_ADJUSTMENT (width_adj));
	gtk_signal_connect (
		GTK_OBJECT (width_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_width_change),
		sensors_app);
	gtk_widget_show (width);
	gtk_box_pack_start (GTK_BOX (hbox14), width, TRUE, TRUE, 5);
	gtk_widget_set_usize (width, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (width), 0);
	
	hbox15 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox15);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox15, TRUE, TRUE, 0);
	
	txt_height = gtk_label_new ("Height:");
	gtk_widget_show (txt_height);
	gtk_box_pack_start (GTK_BOX (hbox15), txt_height, TRUE, TRUE, 0);
	
	height_adj = gtk_adjustment_new (properties->height, 30, 80, 1, 1, 1);
	height = gtk_hscale_new (GTK_ADJUSTMENT (height_adj));
	gtk_signal_connect (
		GTK_OBJECT (height_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_height_change),
		sensors_app);
	gtk_widget_show (height);
	gtk_box_pack_start (GTK_BOX (hbox15), height, TRUE, TRUE, 5);
	gtk_widget_set_usize (height, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (height), 0);
	
	hbox16 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox16);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox16, TRUE, TRUE, 0);
	
	txt_labley = gtk_label_new ("Label Y:");
	gtk_widget_show (txt_labley);
	gtk_box_pack_start (GTK_BOX (hbox16), txt_labley, TRUE, TRUE, 0);
	
	label_y_adj = gtk_adjustment_new (properties->name_ypos, 0, 80, 1, 1, 1);
	label_y_pos = gtk_hscale_new (GTK_ADJUSTMENT (label_y_adj));
	gtk_signal_connect (
		GTK_OBJECT (label_y_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_label_y_change),
		sensors_app);
	gtk_widget_show (label_y_pos);
	gtk_box_pack_start (GTK_BOX (hbox16), label_y_pos, TRUE, TRUE, 5);
	gtk_widget_set_usize (label_y_pos, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (label_y_pos), 0);
	
	hbox17 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox17);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox17, TRUE, TRUE, 0);
	
	txt_valuey = gtk_label_new ("Value Y:");
	gtk_widget_show (txt_valuey);
	gtk_box_pack_start (GTK_BOX (hbox17), txt_valuey, TRUE, TRUE, 0);
	
	value_y_adj = gtk_adjustment_new (properties->value_ypos, 0, 80, 1, 1, 1);
	value_y_pos = gtk_hscale_new (GTK_ADJUSTMENT (value_y_adj));
	gtk_signal_connect (
		GTK_OBJECT (value_y_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_value_y_change),
		sensors_app);	
	gtk_widget_show (value_y_pos);
	gtk_box_pack_start (GTK_BOX (hbox17), value_y_pos, TRUE, TRUE, 5);
	gtk_widget_set_usize (value_y_pos, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (value_y_pos), 0);
	
	hbox18 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox18);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox18, TRUE, TRUE, 0);
	
	txt_unitsy = gtk_label_new ("Units Y:");
	gtk_widget_show (txt_unitsy);
	gtk_box_pack_start (GTK_BOX (hbox18), txt_unitsy, TRUE, TRUE, 0);
	
	units_y_adj = gtk_adjustment_new (properties->units_ypos, 0, 80, 1, 1, 1);
	units_y_pos = gtk_hscale_new (GTK_ADJUSTMENT (units_y_adj));
	gtk_signal_connect (
		GTK_OBJECT (units_y_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_units_y_change),
		sensors_app);
	gtk_widget_show (units_y_pos);
	gtk_box_pack_start (GTK_BOX (hbox18), units_y_pos, TRUE, TRUE, 5);
	gtk_widget_set_usize (units_y_pos, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (units_y_pos), 0);
	
	frame10 = gtk_frame_new ("Updates");
	gtk_widget_show (frame10);
	gtk_box_pack_start (GTK_BOX (vbox4), frame10, TRUE, TRUE, 5);
	
	hbox27 = gtk_hbox_new (FALSE, 5);
	gtk_widget_show (hbox27);
	gtk_container_add (GTK_CONTAINER (frame10), hbox27);
	gtk_container_border_width (GTK_CONTAINER (hbox27), 5);
	
	txt_update_interval = gtk_label_new ("Update\nInterval:");
	gtk_widget_show (txt_update_interval);
	gtk_box_pack_start (GTK_BOX (hbox27), txt_update_interval, TRUE, TRUE, 0);
	
	interval_adj = gtk_adjustment_new ((properties->speed)/1000, 2, 15, 1, 1, 1);
	update_interval = gtk_hscale_new (GTK_ADJUSTMENT (interval_adj));
	gtk_signal_connect (
		GTK_OBJECT (interval_adj),
		"value_changed",
		GTK_SIGNAL_FUNC (on_interval_change),
		sensors_app);
	gtk_widget_show (update_interval);
	gtk_box_pack_start (GTK_BOX (hbox27), update_interval, TRUE, TRUE, 5);
	gtk_widget_set_usize (update_interval, 150, -2);
	gtk_scale_set_digits (GTK_SCALE (update_interval), 0);
	
	return hbox8; 
}

/* ==========================================================================
 * properties_cb
 *
 * Creates and displaylays the propterties dialog.
 * =========================================================================*/
void properties_cb ( AppletWidget *widget, gpointer data)
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	GtkWidget       *page_one;
	
	/* if the window already exists, just show it */
	if (sensors_app->property_box)
	{
		gdk_window_raise(sensors_app->property_box->window);
		return;
	}
	
	/* first, make a copy of the properties to work on - from the stored props */
	load_properties(
		APPLET_WIDGET(sensors_app->my_applet)->privcfgpath, 
		&sensors_app->temporary_properties );
	
	/* build the Gnome Property Box */
	sensors_app->property_box = gnome_property_box_new();
	gtk_window_set_title(GTK_WINDOW(sensors_app->property_box), _("GnomeSensors settings"));
	gtk_window_set_policy(GTK_WINDOW(sensors_app->property_box), FALSE, FALSE, TRUE);
	
	/* page 1 - simple controls */
	page_one = create_first_page(sensors_app);
	gtk_notebook_append_page (
		GTK_NOTEBOOK(GNOME_PROPERTY_BOX(sensors_app->property_box)->notebook),
		page_one, 
		gtk_label_new (_("General") ));
	
	/* some callbacks - who can live without 'em? */
	gtk_signal_connect( 
		GTK_OBJECT(sensors_app->property_box),
		"apply", 
		GTK_SIGNAL_FUNC(on_pb_apply_clicked),
		sensors_app );
	
	gtk_signal_connect( 
		GTK_OBJECT(sensors_app->property_box),
		"destroy", 
		GTK_SIGNAL_FUNC(on_destroy_properties), 
		sensors_app );
	
	/* FIXME - dont forget the help callback. What about ok and cancel?*/
	
	gtk_widget_show_all(sensors_app->property_box);
}


/* ***************************************************************************************************************
 *  The little docky display
 * **************************************************************************************************************/


/* ==========================================================================
 * sensors_resize
 * =========================================================================*/
void sensors_resize (GnomeSensorsApp *sensors_app)
{
	gtk_widget_set_usize(
		sensors_app->display, 
		sensors_app->properties.width,
		sensors_app->properties.height);
	
	gdk_imlib_render(sensors_app->BackgroundPixmap,
		sensors_app->properties.width,
		sensors_app->properties.height);
	
	sensors_app->backpixmap = gdk_imlib_copy_image(sensors_app->BackgroundPixmap);	
}

void sensors_change_fonts(GnomeSensorsApp *sensors_app)
{
	/* try to get some nice fonts - use default if not available */	
	sensors_app->label_font = gdk_font_load (sensors_app->properties.label_font);
	if (sensors_app->label_font == NULL)
	{
		sensors_app->label_font = gdk_font_load(sensors_app->properties.default_font);
	}
	
	sensors_app->value_font = gdk_font_load (sensors_app->properties.value_font);
	if (sensors_app->value_font == NULL) 
	{
		sensors_app->value_font = gdk_font_load(sensors_app->properties.default_font);
	}
	
	sensors_app->units_font = gdk_font_load (sensors_app->properties.units_font);
	if (sensors_app->units_font == NULL) 
	{
		sensors_app->units_font = gdk_font_load(sensors_app->properties.default_font);
	}
}

/* ==========================================================================
 * center_string
 * 
 * plot the string in the center of the widget 
 * =========================================================================*/
void center_string( GdkGC *gc, GdkFont *font, char *str, int y, int total_width)
{
	gint length;
	gint xpos;
	
	length = gdk_string_width(font, str);	
	xpos = (length>total_width) ? 0 : (total_width - length)/2;
	gdk_draw_string(sensors_app->pixmap, font, gc, xpos, y, str);
}

/* ==========================================================================
 * draw
 * 
 * draw the panel widget - it's square, so orientation is no issue 
 * =========================================================================*/
void draw(GnomeSensorsApp *sensors_app)
{	
	/* we need data to plot */
	if ((sensors_app->all_sensors.num_sensors == 0) || 
			(sensors_app->current_sensor_displayed >= sensors_app->all_sensors.num_sensors))
	{
		get_sensor_data(&sensors_app->all_sensors);
		sensors_app->current_sensor_displayed = 0;
	}	
	
	fetch_sensor_for_display(
		&sensors_app->all_sensors, 
		sensors_app->current_sensor_displayed, 
		sensors_app->sensor_name, 
		sensors_app->sensor_value, 
		sensors_app->sensor_units);
	
	/* set the pixmap background */
	gdk_draw_pixmap(sensors_app->pixmap,
		sensors_app->gc,
		sensors_app->backpixmap,
		0, 0, 0, 0, -1, -1);
	
	/* draw text strings */
	gdk_gc_set_foreground( sensors_app->gc, &sensors_app->tcolor );
	
	center_string( 
		sensors_app->gc, 
		sensors_app->label_font, 
		sensors_app->sensor_name, 
		sensors_app->properties.name_ypos,
		sensors_app->properties.width);
	
	center_string( 
		sensors_app->gc, 
		sensors_app->value_font, 
		sensors_app->sensor_value, 
		sensors_app->properties.value_ypos, 
		sensors_app->properties.width);
	
	center_string( 
		sensors_app->gc, 
		sensors_app->units_font, 
		sensors_app->sensor_units, 
		sensors_app->properties.units_ypos,
		sensors_app->properties.width);
	
	/* copy the pixmap into the real window */
	gdk_draw_pixmap(sensors_app->display->window,
		sensors_app->display->style->fg_gc[GTK_WIDGET_STATE(sensors_app->display)],
		sensors_app->pixmap,
		0, 0,
		0, 0,
		sensors_app->display->allocation.width,
		sensors_app->display->allocation.height);
}

/* ==========================================================================
 * setup_colors
 * =========================================================================*/
void setup_colors(GnomeSensorsApp *sensors_app)
{
	GdkColormap *colormap;
	
	colormap = gtk_widget_get_colormap(sensors_app->display);
	
	gdk_color_parse(sensors_app->properties.tcolor, &sensors_app->tcolor);
	gdk_color_alloc(colormap, &sensors_app->tcolor);
}

/* ==========================================================================
 * create_gc
 * =========================================================================*/
void create_gc(GnomeSensorsApp *sensors_app)
{
	sensors_app->gc = gdk_gc_new( sensors_app->display->window );
	gdk_gc_copy( sensors_app->gc, sensors_app->display->style->white_gc );
}

/* ==========================================================================
 * start_timer
 * =========================================================================*/
void start_timer( GnomeSensorsApp *sensors_app )
{
	if( sensors_app->timer_index != -1 )
	{
		gtk_timeout_remove(sensors_app->timer_index);
	}	
	
	sensors_app->timer_index = gtk_timeout_add(
			sensors_app->properties.speed, 
			(GtkFunction)update_values, 
			sensors_app);
}

/* ==========================================================================
 * update_values
 * =========================================================================*/
void update_values(GnomeSensorsApp *sensors_app)
{
	sensors_app->current_sensor_displayed++;
	draw(sensors_app);
}

/* ==========================================================================
 * applet_change_orient
 * =========================================================================*/
static void applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
{
	sensors_resize(sensors_app);
}

/* ==========================================================================
 * sensors_configure
 * =========================================================================*/
static gint sensors_configure(GtkWidget *widget, GdkEventConfigure *event, GnomeSensorsApp *sensors_app)
{
	sensors_app->pixmap = gdk_pixmap_new( 
			widget->window,
			widget->allocation.width,
			widget->allocation.height,
			gtk_widget_get_visual(sensors_app->display)->depth );
	
	if (!sensors_app->gc)
	{
		create_gc(sensors_app);
	}	
	
	draw(sensors_app);
	return TRUE;
} 

/* ==========================================================================
 * sensors_expose
 * =========================================================================*/
static gint sensors_expose(GtkWidget *widget, GdkEventExpose *event, GnomeSensorsApp *sensors_app)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		sensors_app->pixmap,
		event->area.x, event->area.y,
		event->area.x, event->area.y,
		event->area.width, event->area.height);
	return FALSE;
}


/* ==========================================================================
 * sensors_clicked_cb
 * =========================================================================*/
static gint sensors_clicked_cb(
	GtkWidget * widget, 
	GdkEventButton * e, 
	GnomeSensorsApp *sensors_app) 
{	
	/* Ignore buttons 2 and 3 */
	if (e->button != 1) {		
		return FALSE; 
	}
	
	update_values(sensors_app);	
	return TRUE; 
}

/* ==========================================================================
 * start_monitor_sensors_cb
 * =========================================================================*/
void start_monitor_sensors_cb(
	AppletWidget *widget,
	gpointer data)
{
	gnome_execute_shell(NULL, "MonitorSensors");
}

/* ==========================================================================
 * destroy_about
 * =========================================================================*/
void destroy_about(
	AppletWidget *widget,
	gpointer data )
{
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	sensors_app->aboutbox=NULL;
}

/* ==========================================================================
 * about_cb
 * =========================================================================*/
void about_cb (
	AppletWidget *widget,
	gpointer data)
{
	static const char *authors[] = { "Brendon Humphrey <brendy@swipnet.se>", NULL };
	GnomeSensorsApp *sensors_app = (GnomeSensorsApp *)data;
	
	if(sensors_app->aboutbox) {
		gtk_widget_show(sensors_app->aboutbox);
		gdk_window_raise(sensors_app->aboutbox->window);
		return;
	}
	
	sensors_app->aboutbox = gnome_about_new (_("The GNOME Sensors applet"),
			_(VERSION),
			_("(C) 1999 Brendon Humphrey"),
			(const char **)authors,
			_("This applet uses the lm_sensors kernel "
				"modules to displaylay the state of "
				"the various sensor chips that "
				"may be present in your computer. "
				"It does not react to alarms or "
				"allow configuration of sensor limits. "
				"You should use /etc/sensors.conf for that."),
			NULL);
	
	gtk_signal_connect(GTK_OBJECT(sensors_app->aboutbox),"destroy",
		GTK_SIGNAL_FUNC(destroy_about),sensors_app);
	gtk_widget_show (sensors_app->aboutbox);
	
	return;
}

/* ==========================================================================
 * sensors_widget
 * =========================================================================*/
GtkWidget *sensors_widget(GnomeSensorsApp *sensors_app)
{
	GtkWidget *frame;
	GtkWidget *box;
	GtkWidget *event_box;
	
	box = gtk_vbox_new(FALSE, FALSE);
	gtk_widget_show(box);
	
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type( GTK_FRAME(frame), sensors_app->properties.look?GTK_SHADOW_OUT:GTK_SHADOW_IN );
	
	sensors_app->display = gtk_drawing_area_new();
	gtk_signal_connect( GTK_OBJECT(sensors_app->display), "expose_event",
		(GtkSignalFunc)sensors_expose, sensors_app);
	gtk_signal_connect( GTK_OBJECT(sensors_app->display),"configure_event",
		(GtkSignalFunc)sensors_configure, sensors_app);
	gtk_widget_set_events( sensors_app->display, GDK_EXPOSURE_MASK );
	
	gtk_box_pack_start_defaults( GTK_BOX(box), sensors_app->display );
	
	event_box = gtk_event_box_new();
	gtk_widget_show(event_box);
	gtk_widget_set_events(event_box, GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(event_box), "button_press_event",
		GTK_SIGNAL_FUNC(sensors_clicked_cb), sensors_app);
	
	gtk_container_add( GTK_CONTAINER(event_box), box );
	gtk_container_add( GTK_CONTAINER(frame), event_box);
	
	sensors_resize(sensors_app);
	start_timer(sensors_app);
	gtk_widget_show_all(frame);
	
	return frame;	
}

/* ==========================================================================
 * applet_save_session
 * =========================================================================*/
static gint applet_save_session(
	GtkWidget *widget, 
	char *privcfgpath, 
	char *globcfgpath, 
	GnomeSensorsApp *sensors_app)
{
	save_properties(privcfgpath,&sensors_app->properties);
	return FALSE;
}


/* ***************************************************************************************************************
 *  Lets get the show on the road...
 * **************************************************************************************************************/


/* ==========================================================================
 * main
 * =========================================================================*/
int main(int argc, char **argv)
{
	GtkWidget *applet;
	int result;
	GdkImlibBorder border = {2,2,2,2};
	
	/* get memory for this applets resources */
	sensors_app = g_new0(GnomeSensorsApp,1);
	sensors_app->timer_index=-1;
	sensors_app->first_time=1;
	
	/* Initialize the i18n stuff */
	/*bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);*/
	
	/* initialise the sensors package (report on failure)*/
	sensors_app->sensors_config_file=(FILE *)fopen("/etc/sensors.conf", "r");
	if((result = sensors_init(sensors_app->sensors_config_file)))
	{
		fprintf(stderr,"%s\n",sensors_strerror(result));
		exit(1);
	}
	
	sensors_app->current_sensor_displayed = 0;
	sensors_app->all_sensors.num_sensors = 0;
	initialise_chips();
	
	/* initialise graphics stuff */
	gdk_init(&argc, &argv);
	gdk_imlib_init();
	gtk_widget_push_visual(gdk_imlib_get_visual());
	gtk_widget_push_colormap(gdk_imlib_get_colormap());
	sensors_app->bg_pixmap_path = gnome_unconditional_pixmap_file("GnomeSensorsBg.xpm");
	sensors_app->BackgroundPixmap=gdk_imlib_load_image(sensors_app->bg_pixmap_path);
	gdk_imlib_set_image_border(sensors_app->BackgroundPixmap, &border);
	
	assert(sensors_app->BackgroundPixmap);
	
	/* initialise the applet */
	applet_widget_init(
		"sensors_applet", 
		NULL, 
		argc, 
		argv,
		NULL, 
		0, 
		NULL);
	applet = applet_widget_new("gnome_sensors_applet");
	if (!applet)
		g_error("Can't create applet!\n");
	
	sensors_app->my_applet = applet;
	load_properties(APPLET_WIDGET(applet)->privcfgpath, &sensors_app->properties);
	
	gdk_imlib_render(sensors_app->BackgroundPixmap,
		sensors_app->properties.width,
		sensors_app->properties.height);                    
	sensors_app->backpixmap = gdk_imlib_copy_image(sensors_app->BackgroundPixmap);
	
	assert(sensors_app->backpixmap);
	
	sensors_change_fonts(sensors_app);
	
	sensors_app->sensors = sensors_widget(sensors_app);
	applet_widget_add( APPLET_WIDGET(applet), sensors_app->sensors );
	gtk_widget_show(applet);
	setup_colors(sensors_app);	
	
	/* connect signals to applet */	
	gtk_signal_connect(
		GTK_OBJECT(applet),
		"change_orient",
		GTK_SIGNAL_FUNC(applet_change_orient),
		sensors_app);
	
	gtk_signal_connect(
		GTK_OBJECT(applet),
		"save_session",
		GTK_SIGNAL_FUNC(applet_save_session),
		sensors_app);
	
	/* add menu items...*/
	applet_widget_register_stock_callback(
		APPLET_WIDGET(applet),
		"properties",
		GNOME_STOCK_MENU_PROP,
		_("Properties..."),
		properties_cb,
		sensors_app);	
	
	applet_widget_register_stock_callback(
		APPLET_WIDGET(applet),
		"about",
		GNOME_STOCK_MENU_ABOUT,
		_("About..."),
		about_cb,
		(gpointer)sensors_app);
	
	/*
	 * the presense of this menuitem is conditional on configure 
	 * defining HAS_MONITORSENSORS 
	 */
	
#ifdef HAS_MONITORSENSORS
	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
		"run_monitorsensors",
		GNOME_STOCK_MENU_EXEC,
		_("Run MonitorSensors..."),
		start_monitor_sensors_cb, NULL);
#endif
	
	/* execute!*/
	update_values(sensors_app);
	applet_widget_gtk_main();
	
	/* clean up*/
	fclose(sensors_app->sensors_config_file);
	
	return 0;
}
