/*
 * GNOME hardware sensors panel applet
 * (C) 1999 The Free Software Foundation
 *
 * Displays the state of hardware sensors in your machine.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *                                
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *           
 * Authors: 	Brendon Humphrey (brendy@swipnet.se)
 *
 * This applet is based on the diskusage applet and the 'sensors'
 * application that is included in the lm_sensors package.
 *
 */

#ifndef CHIPS_H
#define CHIPS_H

#ifdef __cplusplus
extern "C" {
#endif
	
#define MAX_BUFFER_SIZE 50
	
	/* ==========================================================================
 	 * Type declarations
 	 * =========================================================================*/
	typedef enum _chips
	{
		LM75 = 0,
		ADM1021,     /* adm1021, max1617, max1617a, thmc10*/
		ADM9240,     /* adm9240, ds1780, lm81*/
		LM78,          /* lm78, lm79, lm78-j */
		SIS5595,
		LM80,
		GL518,
		GL520,
		W83781D,    /* w83781d, w83782d, w83783s, w83627hf*/
		MAXILIFE    /* maxilife-cg, maxilife-co, maxilife-as */
	} known_chips;
	
	typedef enum _units
	{
		VOLTS=0,
		CELSIUS,
		FARANHEIGHT,
		RPM,
		MHZ
	} sensor_units;
	
	typedef struct _sensor 
	{
		char	alarm;
		char name[50];
		double min;
		double max;
		double now;
		sensor_units units;
	} sensor_reading;
	
	
	typedef struct _machine
	{
		int num_sensors;
		sensor_reading *sensors;
	} machine;
	
	/* ==========================================================================
           * initialise_chips
	 * 
	 *  
           * =========================================================================*/
	void initialise_chips();
	
	/* ==========================================================================
           * get_sensor_data
           * =========================================================================*/
	void get_sensor_data( machine *all_sensors );
	
	/* ==========================================================================
           * fetch_sensor_for_display
           * =========================================================================*/
	void fetch_sensor_for_display( machine *all_sensors, int index, char *name, char *value, char *units);
	
#ifdef __cplusplus
}
#endif

#endif
